var RetractionDestructForm = function()
{
	var rd = {};
	var patientInfo;
	var caseIsValid = false;
	var debug = 0;
	
	rd.initPage = function()
	{
		// Step 1
		Events.addEventHandler('step-1', 'wizard-validate', rd.validateStep1);
		Events.addEventHandler('caseName', 'blur', rd.caseNameOnChange);
		Events.addEventHandler('caseName', 'keypress', rd.keyPressCheck);
		//Events.addEventHandler('caseName', 'keypress', Wizard.goNextOnTabOrEnter);
		
		// Step 2
		Events.addEventHandler('step-2', 'wizard-initialize', rd.initializeStep2);
		Events.addEventHandler('step-2', 'wizard-validate', rd.validateStep2);

		// Navigation
		Buttons.addClickHandler('gocancel', Wizard.cancelWizard);
		Buttons.addClickHandler('gorestart', Wizard.restartWizard);
		Buttons.addClickHandler('gonext', Wizard.goNextOnClick);
		Buttons.addClickHandler('goregister', Wizard.goRegister);		

		// Final registration
		Events.addEventHandler('wizard', 'wizard-submit', rd.submit);
		
		Doc.show('step-1');
		Doc.show('gonext');
	}

	/**
		Check case name if key pressed if Tab or Return/Enter.
	*/
	rd.keyPressCheck = function(event)
	{
		// Check case name if key pressed if Tab or Return/Enter
		if (event.keyCode == 9 || event.keyCode == 13)
		{
			rd.caseNameOnChange(event);
		}
	}
	
	/**
		Check that the case number is valid.
	*/
	rd.caseNameOnChange = function(event)
	{
		var frm = document.forms['reggie'];
		var caseName = frm.caseName.value.toUpperCase();
		caseIsValid = false;
		if (caseName == '')
		{
			Wizard.setInputStatus('caseName', 'invalid', 'Missing');
			frm.caseName.focus();
			return;
		}
		else if (!Reggie.isValidCaseName(caseName, false)) // 'C' suffix is not used
		{
			rd.validatePersonalNumber(caseName);
			if (pnrIsValid)
			{
				caseIsValid = true;
			}
			else
			{
				// Entered number is neither a valid case name or personal number
				// Try to give as appropriate error message as possible
				if (caseName.length < 7 || pnrErrorMsg == null || pnrErrorMsg == '')
				{
					// The entered number is probably a faulty case name
					Wizard.setInputStatus('caseName', 'invalid', 'Only 7-digits name is allowed .');
				}
				else
				{
					Wizard.setInputStatus('caseName', 'invalid', pnrErrorMsg);
				}
				frm.caseName.focus();
				return;
			}
		}

		frm.caseName.value = caseName;

		// Check that the patient item is of type "Retract" or "RetroNo"		
		var url = '../Retraction.servlet?ID='+App.getSessionId();
		url += '&cmd=GetCaseInfo';
		url += '&caseName=' + encodeURIComponent(caseName);
		Wizard.asyncJsonRequest(url, rd.onCaseInfoLoaded);
	}

	rd.onCaseInfoLoaded = function(response)
	{
		var frm = document.forms['reggie'];
		
		// Get biosource information from the AJAX response
		caseInfo = response.caseInfo;
		patientInfo = response.patient;

		if (patientInfo)
		{
			var patientName = patientInfo.name;
			var retractStatus = patientInfo.retractStatus;
			// Check if patient name does not start with "Retract" or "RetroNo"
			if (patientName.indexOf('Retract') != 0 && patientName.indexOf('RetroNo') != 0)
			{
				Wizard.setInputStatus('caseName', 'invalid', 'Patient name \"' + patientName + '\" does not start with \"Retract\" or \"RetroNo\".');
				return;
			}
			else if (retractStatus == null || retractStatus != 'ReadyForDestruction')
			{
				Wizard.setInputStatus('caseName', 'invalid', 'Patient name \"' + patientName + '\" has \"RetractStatus\" annotation value set to \"'+retractStatus+'\", not \"ReadyForDestruction\".');
				return;
			}
			else
			{
				caseIsValid = true;
				Wizard.setInputStatus('caseName', 'valid');
				Wizard.goNext(true);
				return;
			}
		}
	}
	
	/**
		Check that the case number is valid.
	*/
	rd.validateStep1 = function(event)
	{
		if (event.detail.auto)
		{
			if (!caseIsValid) 
			{
				event.preventDefault();
			}
		}
		else
		{
			if (!caseIsValid) event.preventDefault();
		}
	}

	rd.validatePersonalNumber = function(pnr)
	{
		pnrIsValid = false;
		pnrIsAcceptable = true;
		pnrErrorMsg = null;
		
		var pnrLen = pnr.length;
		if (pnrLen != 12 && pnrLen != 10)
		{
			Wizard.setInputStatus('caseName', 'warning', pnrLen < 12 ? 'Personal number too short' : 'Personal number too long');
			if (pnrLen < 12)
			{
				pnrErrorMsg = 'Personal number too short';
			}
			else
			{
				pnrErrorMsg = 'Personal number too long';
			}
			return;
		}

		if (!pnr.match(/^\d+$/))
		{
			if (!pnr.match(/^\d+[A-Za-z]$/))
			{
				return;
			}
		}
		
		var datePart = pnr.substring(0, pnrLen - 4); // Remove last four digits
		var dateFormat = pnrLen == 10 ? 'yyMMdd' : 'yyyyMMdd';
		var isValidDate = Dates.isDate(datePart, dateFormat);
		if (!isValidDate)
		{
			var dayInMonth = parseInt(datePart.substring(-2), 10); // Last two characters
			if (dayInMonth > 60)
			{
				// 'Samordningsnummer' has day-in-month + 60; check this
				dayInMonth -= 60;
				var tmpPnr = pnr.substring(0, pnrLen - 6); // First 4 or 6 digits is year+month
				if (dayInMonth < 10) tmpPnr += '0';
				tmpPnr += dayInMonth;
				isValidDate = Dates.isDate(tmpPnr, dateFormat);
			}
		}
		if (!isValidDate)
		{
			Wizard.setInputStatus('caseName', 'warning', 'Personal number: Not a valid date');
			pnrErrorMsg = 'Personal number: Not a valid date';
			return;
		}
		
		if (!pnr.match(/^\d+[A-Za-z]$/))
		{
			if (!Reggie.personalNumberControlDigitCheck(pnr.substr(pnrLen == 10 ? 0 : 2)))
			{
				Wizard.setInputStatus('caseName', 'warning', 'Personal number: Invalid control digit');
				pnrErrorMsg = 'Personal number: Invalid control digit';
				return;
			}
		}

		Wizard.setInputStatus('caseName', 'valid');
		pnrIsValid = true;
	}

	/**
		Load information about the given case.
	*/
	rd.initializeStep2 = function(event)
	{
		var frm = document.forms['reggie'];
		var caseName = frm.caseName.value;

		var url = '../Retraction.servlet?ID='+App.getSessionId();
		url += '&cmd=GetCaseInfoBeforeDestruction';
		url += '&caseName=' + encodeURIComponent(caseName);
		
		Wizard.showLoadingAnimation('Loading case information...');
		Wizard.asyncJsonRequest(url, rd.caseInfoLoaded);
	}

	/**
		Initalize the second step based on the information we have about the case.
	*/
	rd.caseInfoLoaded = function(response)
	{
		var frm = document.forms['reggie'];
		var caseName = frm.caseName.value;
		patientInfo = response.patient;
		var patientName = patientInfo ? patientInfo.name : '';

		if (!(patientName.indexOf('Retract') == 0 || patientName.indexOf('RetroNo') == 0))
		{
			Wizard.setFatalError('Patient name "' + patientName + '" does not start with "Retract" or "RetroNo".');
			return;
		}
		
		// Sample Items Header row
		var html = '<table id="retractionItemsTable2">';
		html += '<thead class="bg-filled-100">';
		html += '<tr>';
		html += '<th></th>';
		html += '<th>Item</th>';
		html += '<th>Name</th>';
		html += '<th>Action</th>';
		html += '</tr>';
		html += '</thead>';
		
		html += '<tbody>';
		
		var items = response.items;
		var allCanBeDeleted = true;
		for (var i = 0; i < items.length; i++)
		{
			var item = items[i];
			var comment = item.comment;
			if (comment && comment == 'To be deleted')
			{
				comment = '<b>' + comment + '</b>';
			}
			if (!item.deletePermission)
			{
				allCanBeDeleted = false;
				comment = '<b>No delete permission!</b>';
			}
			html += '<tr class="'+(!item.deletePermission ? 'no-delete' : 'highlight')+'">';
			html += '<th class="index">' + (i+1) + '</th>';
			html += '<td class="dottedleft">' + Strings.encodeTags(item.type) + '</td>';
			html += '<td>' + Strings.encodeTags(item.name) + '</td>';
			html += '<td class="dottedleft">' + comment + '</td>';
			html += '</tr>';
		}
		
		html += '</tbody>';
		html += '</table>';
		Doc.element('retractTable').innerHTML = html;

		Wizard.setCurrentStep(2);
		Doc.show('gocancel');
		Doc.show('step-1');
		if (allCanBeDeleted)
		{
			Doc.show('goregister');
		}
	}

	rd.validateStep2 = function(event)
	{
		// Always valid
	}
		
	rd.submit = function()
	{
		var frm = document.forms['reggie'];
		var caseName = frm.caseName.value;

		var retractionInfo = {};
		retractionInfo.caseName = caseName;

		var submitInfo = {};
		submitInfo.retractionInfo = retractionInfo;

		var url = '../Retraction.servlet?ID=' + App.getSessionId();
		url += '&cmd=RegisterDestruction';

		Wizard.showLoadingAnimation('Performing registration...');
		Wizard.asyncJsonRequest(url, rd.submissionResults, 'POST', JSON.stringify(submitInfo));
	}
	
	rd.submissionResults = function(response)
	{
		Wizard.showFinalMessage(response.messages);
		Doc.show('gorestart');
	}

	return rd;
}();

Doc.onLoad(RetractionDestructForm.initPage);

